package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.interfaces.PermissionService;
import com.bocloud.sms.model.*;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 权限控制器
 *
 * @author tyl
 */
@RestController
@Tag(name = "权限管理")
@RequestMapping("/v1/permissions")
public class PermissionController {

    private final PermissionService permissionService;

    public PermissionController(PermissionService permissionService) {
        this.permissionService = permissionService;
    }

    /**
     * 查询权限树
     *
     * @param parentId 父权限id
     * @param tenant   是否为租户权限
     * @return 权限树
     */
    @Operation(summary = "查询权限列表")
    @GetMapping("/tree")
    public GeneralResult<List<PermissionBean>> list(@RequestParam(value = Common.PARENTID, required = false) Long parentId, @RequestParam(value = "category", required = false) String category, @RequestParam(value = "tenant", required = false) Boolean tenant) {
        return permissionService.list(parentId, category, tenant);
    }

    /**
     * 查询图标
     *
     * @return 图标列表
     */
    @Operation(summary = "查询图标")
    @GetMapping("/icons")
    public GeneralResult<List<PermissionIcon>> listIcon(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.listIcon(requestContext);
    }

    /**
     * 查询所有权限父节点
     *
     * @return 权限列表
     */
    @Operation(summary = "查询父节点")
    @GetMapping("/parents")
    public GeneralResult<List<PermissionParentsBean>> listParents(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.listParents(requestContext);
    }

    /**
     * 添加权限
     *
     * @param permissionModel 权限
     * @param requestContext  上下文
     * @return 添加结果
     */
    @Operation(summary = "添加权限")
    @PostMapping
    public Result create(@RequestBody PermissionModel permissionModel, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.create(permissionModel, requestContext);
    }

    /**
     * 修改权限
     *
     * @param id              权限id
     * @param permissionModel 权限
     * @param requestContext  上下文
     * @return 修改结果
     */
    @Operation(summary = "权限修改")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody PermissionModel permissionModel, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.modify(id, permissionModel, requestContext);
    }

    /**
     * 移除权限
     *
     * @param id             权限id
     * @param requestContext 上下文
     * @return 移除结果
     */
    @Operation(summary = "删除权限")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.remove(id, requestContext);
    }

    /**
     * 查看权限详细信息
     *
     * @param id 权限id
     * @return 权限
     */
    @Operation(summary = "查询权限详细信息")
    @GetMapping("/{id}")
    public GeneralResult<Permission> detail(@PathVariable(value = Common.ID) Long id) {
        return permissionService.detail(id);
    }

    /**
     * 按类型查询权限列表
     *
     * @param parentId 父权限id
     * @param tenant   是否为租户权限
     */
    @Operation(summary = "查询权限列表")
    @GetMapping
    public GeneralResult<List<PermissionBean>> listPermissions(@RequestParam(value = Common.PARENTID, required = false) Long parentId, @RequestParam(value = "category", required = false) String category, @RequestParam(value = "tenant", required = false) Boolean tenant) {
        return permissionService.list(parentId, category, tenant);
    }


    /**
     * 按字典查询权限列表
     *
     * @param condition 字典值
     */
    @Operation(summary = "字典查询权限列表")
    @GetMapping("/condition")
    public GeneralResult<List<Permission>> listPermissionsByCondition(@RequestParam(value = Common.CONDITION, required = false) String condition) {
        return permissionService.listPermissionsByCondition(condition);
    }

    /**
     * 根据ServiceId获取服务菜单权限信息
     *
     * @param serviceId 服务id
     * @return 菜单权限
     */
    @Operation(summary = "查询服务菜单权限")
    @GetMapping("/info")
    public GeneralResult<Permission> getServicePermissions(@RequestParam(value = "serviceId", required = false) Long serviceId) {
        return permissionService.getServicePermission(serviceId);
    }

    /**
     * 查询服务模块api权限列表
     *
     * @return 权限列表
     */
    @Operation(summary = "查询服务模块API权限列表")
    @GetMapping("/modules")
    public GeneralResult<JSONArray> getApiList() {
        return permissionService.getApiList();
    }

    /**
     * 根据服务code查询服务模块API权限列表
     *
     * @param serviceCodes
     * @return
     */
    @Operation(summary = "根据服务code查询服务模块API权限列表")
    @GetMapping("/modules/services")
    public GeneralResult<JSONArray> getApiList(@RequestParam(value = "serviceCodes") String serviceCodes) {
        List<String> codes = JSONArray.parseArray(serviceCodes).toJavaList(String.class);
        if (!codes.isEmpty()) {
            return permissionService.getApiList(codes);
        }
        return new GeneralResult<>(false, "参数格式错误");
    }

    /**
     * 插件绑定权限
     *
     * @param permissionPlugins 权限插件
     * @param requestContext    上下文
     * @return 权限列表
     */
    @Operation(summary = "插件绑定权限")
    @PutMapping("/binding/plugins")
    public Result bindingPlugins(@RequestBody PermissionPlugins permissionPlugins, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return permissionService.bindingPlugins(permissionPlugins.getPermissionIdList(), permissionPlugins.getPluginsCode(), permissionPlugins.getTenant(), requestContext);
    }
}
