package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.model.DictionaryBean;
import com.bocloud.sms.repository.DictionaryRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 字典服务实现类
 *
 * @author dmw
 */
@RequiredArgsConstructor
@Service("dictService")
@Slf4j
public class DictServiceImpl implements DictService {

    private final DictionaryRepository dictionaryRepository;
    private final LockFactory lockFactory;
    private final StringRedisTemplate redisTemplate;

    @Override
    public GeneralResult<List<Dictionary>> list(Long pid) {
        Map<Long, List<Dictionary>> dictionaryMap = dictionaryRepository.list().stream().collect(Collectors.groupingBy(Dictionary::getPid));
        List<Dictionary> dictionaries = dictionaryRepository.dictionaries(pid);
        for (Dictionary dictionary : dictionaries) {
            getChildren(dictionary, dictionaryMap);
        }
        return new GeneralResult<>(true, dictionaries, "查询成功");
    }

    @Override
    public GeneralResult<List<Dictionary>> listChild(String value, Long id) {
        Dictionary dictionary;
        if (null == id) {
            dictionary = dictionaryRepository.queryFistDic(value);
        } else {
            dictionary = dictionaryRepository.queryDic(value, id);
        }
        if (dictionary == null) {
            dictionary = dictionaryRepository.queryDic(value);
        }
        if (null == dictionary) {
            return new GeneralResult<>(true, "未找到【" + value + "】字典");
        }
        List<Dictionary> list = this.dictionaryRepository.dictionaries(dictionary.getId());
        return new GeneralResult<>(true, list, "success");
    }

    @Override
    public GeneralResult<Dictionary> create(Dictionary dictionary) {
        boolean existValue = dictionaryRepository.existValue(dictionary.getPid(), dictionary.getValue());
        if (existValue) {
            return new GeneralResult<>(false, "内容已存在，添加失败");
        }
        dictionaryRepository.save(dictionary);
        // 变更缓存中操作系统名称
        this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_OS_CATEGORY_TASK",
                JSONObject.toJSONString(getAllOsName()));
        this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_DATA_CENTER",
                JSONObject.toJSONString(getAllDataCenter()));
        return new GeneralResult<>(true, dictionary, "保存成功!");
    }

    @Override
    public Result remove(Long id) {
        String path = Dictionary.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Dictionary dictionary = dictionaryRepository.query(id);
            if (null == dictionary) {
                return new GeneralResult<>(false, "数据不存在!");
            }
            Map<Long, List<Dictionary>> dictionaryMap = dictionaryRepository.list().stream().collect(Collectors.groupingBy(Dictionary::getPid));
            List<Long> childrenIds = getChildrenIds(dictionary, dictionaryMap);
            childrenIds.add(dictionary.getId());
            boolean result = dictionaryRepository.batchRemove(childrenIds);
            // 变更缓存中操作系统名称
            this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_OS_CATEGORY_TASK",
                    JSONObject.toJSONString(getAllOsName()));
            this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_DATA_CENTER",
                    JSONObject.toJSONString(getAllDataCenter()));
            return result ? new GeneralResult<>(true, "删除成功!") : new GeneralResult<>(false, "删除失败!");
        }
    }

    private List<Long> getChildrenIds(Dictionary dictionary, Map<Long, List<Dictionary>> dictionaryMap) {
        List<Long> childrenIds = new ArrayList<>();
        List<Dictionary> children = dictionaryMap.get(dictionary.getId());
        if (children != null) {
            for (Dictionary child : children) {
                childrenIds.add(child.getId());
                List<Long> childIds = getChildrenIds(child, dictionaryMap);
                if (!childIds.isEmpty()) {
                    childrenIds.addAll(childIds);
                }
            }
        }
        return childrenIds;
    }

    @Override
    public GeneralResult<Dictionary> modify(DictionaryBean bean) {
        String path = Dictionary.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Dictionary dictionary = dictionaryRepository.query(bean.getId());
            if (null == dictionary) {
                return new GeneralResult<>(false, "数据不存在!");
            }
            BeanUtils.copyProperties(bean, dictionary);
            dictionaryRepository.update(dictionary);
            // 变更缓存中操作系统名称
            this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_OS_CATEGORY_TASK",
                    JSONObject.toJSONString(getAllOsName()));
            this.redisTemplate.opsForValue().set(Dictionary.class.getSimpleName() + "_DATA_CENTER",
                    JSONObject.toJSONString(getAllDataCenter()));
            return new GeneralResult<>(true, dictionary, "修改成功!");
        }
    }

    @Override
    public GeneralResult<List<Dictionary>> childTree(String value) {
        Dictionary dictionary = dictionaryRepository.queryFistDic(value);
        if (dictionary == null) {
            dictionary = dictionaryRepository.queryDic(value);
        }
        Assert.notNull(dictionary, "未找到value为[" + value + "]字典");
        List<Dictionary> result = this.dictionaryRepository.dictionaries(dictionary.getId());
        Map<Long, List<Dictionary>> dictionaryMap =
                dictionaryRepository.list().stream().collect(Collectors.groupingBy(Dictionary::getPid));
        for (Dictionary dic : result) {
            getChildren(dic, dictionaryMap);
        }
        return new GeneralResult<>(true, result, "success");
    }

    private void getChildren(Dictionary dictionary, Map<Long, List<Dictionary>> dicMap) {
        List<Dictionary> children = dicMap.get(dictionary.getId());
        if (null != children) {
            children.sort(Comparator.comparingLong(Dictionary::getWeight));
            for (Dictionary child : children) {
                getChildren(child, dicMap);
            }
            dictionary.setChildren(children);
        }
    }

    @Override
    public List<String> getAllOsName() {
        List<String> list = new ArrayList<>();
        // 查询osNames列表
        Dictionary root = dictionaryRepository.queryDic("OS_CATEGORY_TASK");
        if (null == root) {
            return list;
        }
        List<Dictionary> osCategories = dictionaryRepository.dictionaries(root.getId());
        for (Dictionary osCategory : osCategories) {
            List<Dictionary> children = dictionaryRepository.dictionaries(osCategory.getId());
            children.forEach(dict -> list.add(dict.getValue()));
        }
        return list;
    }

    @Override
    public Map<String, String> getAllDataCenter() {
        Map<String, String> map = new HashMap<>(16);
        // 数据中心相关
        Dictionary root = dictionaryRepository.queryDic("DATACENTER");
        if (null == root) {
            return map;
        }
        List<Dictionary> dcs = dictionaryRepository.dictionaries(root.getId());
        for (Dictionary dc : dcs) {
            getDictionary(dc, map, dc.getValue());
        }
        return map;
    }

    /**
     * 通过内容模型id
     *
     * @param id id
     * @return {@link GeneralResult}<{@link Dictionary}>
     */
    @Override
    public GeneralResult<Dictionary> detail(Long id) {
        Dictionary dictionary = dictionaryRepository.query(id);
        if (dictionary == null) {
            return new GeneralResult<>(false,"查询失败");
        }
        return new GeneralResult<>(true, dictionary, "查询成功");
    }

    /**
     * 循环获取子项放入mao中
     *
     * @param dictionary 字典数据
     * @param map        map
     */
    private void getDictionary(Dictionary dictionary, Map<String, String> map, String key) {
        List<Dictionary> children = dictionaryRepository.dictionaries(dictionary.getId());
        map.put(key, dictionary.getName());
        children.forEach(child -> getDictionary(child, map, key + "_" + child.getValue()));
    }
}