package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.entity.Plugin;
import com.bocloud.sms.interfaces.PermissionService;
import com.bocloud.sms.model.*;
import com.bocloud.sms.repository.PermissionRepository;
import com.bocloud.sms.repository.PluginRepository;
import com.bocloud.sms.repository.RolePermissionRepository;
import com.bocloud.sms.utils.AuthParentsComparator;
import com.bocloud.sms.utils.PermissionComparator;
import com.bocloud.sms.utils.PermitComparator;
import com.google.common.collect.Lists;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 权限抽象Service接口实现类
 *
 * @author tyl
 */
@Service("permissionServiceImpl")
@Slf4j
public class PermissionServiceImpl implements PermissionService {

    private final PermissionRepository permissionRepository;
    private final StringRedisTemplate redisTemplate;
    private final LockFactory lockFactory;
    private final PluginRepository pluginRepository;
    private final RolePermissionRepository rolePermissionRepository;

    public PermissionServiceImpl(PermissionRepository permissionRepository, StringRedisTemplate redisTemplate,
                                 LockFactory lockFactory, PluginRepository pluginRepository,
                                 RolePermissionRepository rolePermissionRepository) {
        this.permissionRepository = permissionRepository;
        this.redisTemplate = redisTemplate;
        this.lockFactory = lockFactory;
        this.pluginRepository = pluginRepository;
        this.rolePermissionRepository = rolePermissionRepository;
    }

    /**
     * 添加权限
     *
     * @param permissionModel
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result create(PermissionModel permissionModel, RequestContext requestContext)
            throws InternalServerException {
        Permission permission = new Permission();
        BeanUtils.copyProperties(permissionModel, permission);
        Assert.notNull(permission, "permission can't be null");
        permission.setId(null);
        if ("api".equals(permission.getCategory())) {
            permission.setPriority(0);
        }
        try {
            permission.setStatus(BaseStatus.NORMAL.name());
            permission.setCreatorId(requestContext.getTarget());
            permissionRepository.save(permission);
            return new Result(true, "添加权限成功");
        } catch (Exception e) {
            log.error("create permission fail:", e);
            throw new InternalServerException("添加权限失败", e, e.getMessage());
        }
    }

    /**
     * 修改权限
     *
     * @param id
     * @param permission     权限信息json串
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(Long id, PermissionModel permission, RequestContext requestContext)
            throws InternalServerException {
        String path = Permission.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Permission existPermission = permissionRepository.query(id);
            Assert.notNull(existPermission, "permission don't exists");
            existPermission.setMenderId(requestContext.getTarget());
            BeanUtils.copyProperties(permission, existPermission, "status", "tenant");
            boolean flag = permissionRepository.update(existPermission);
            return flag ? new Result(true, "修改权限成功") : new Result(false, "修改权限失败");
        } catch (Exception e) {
            log.error("update permission fail:", e);
            throw new InternalServerException("修改权限失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 删除权限
     *
     * @param id             权限id
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id, RequestContext requestContext) throws InternalServerException {
        String path = Permission.class.getSimpleName() + "_" + id;
        int result;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Permission permission = permissionRepository.query(id);
            Assert.notNull(permission, "该权限不存在");
            // 递归查询子权限id
            Map<Long, List<Permission>> permissionMap =
                    permissionRepository.listAll().stream().collect(Collectors.groupingBy(Permission::getParentId));
            List<Long> childrenIds = getChildrenIds(permission, permissionMap);
            childrenIds.add(permission.getId());
            // 删除权限和角色、租户的关联关系
            rolePermissionRepository.unBindRole(childrenIds);
            result = permissionRepository.batchDelete(childrenIds, requestContext.getTarget());
        } catch (Exception e) {
            log.error("remove auth fail:", e);
            throw new InternalServerException("删除权限失败," + e.getMessage(), e, e.getMessage());
        }
        return result > 0 ? new Result(true, "删除权限成功") : new Result(false, "删除权限失败");
    }

    private List<Long> getChildrenIds(Permission permission, Map<Long, List<Permission>> permissionMap) {
        List<Long> childrenIds = new ArrayList<>();
        List<Permission> children = permissionMap.get(permission.getId());
        if (children != null) {
            for (Permission child : children) {
                childrenIds.add(child.getId());
                List<Long> childIds = getChildrenIds(child, permissionMap);
                if (!childIds.isEmpty()) {
                    childrenIds.addAll(childIds);
                }
            }
        }
        return childrenIds;
    }

    /**
     * 查看权限详情
     *
     * @param id 权限id
     * @return
     */
    @Override
    public GeneralResult<Permission> detail(Long id) {
        try {
            Permission permission = permissionRepository.query(id);
            return new GeneralResult<>(true, permission, "查询成功");
        } catch (Exception e) {
            log.error("query permission fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<PermissionParentsBean>> listParents(RequestContext requestContext) {
        try {
            boolean tenant = false;
            // 判断是否是租户（自服务端）权限
            if (null != requestContext.getTenant() && requestContext.getTenant() > 0) {
                tenant = true;
            }
            // 获取根权限节点
            List<Permission> list = permissionRepository.listRoots(tenant);
            List<PermissionParentsBean> parents = new ArrayList<>();
            for (Permission permission : list) {
                PermissionParentsBean parent =
                        new PermissionParentsBean(permission.getId(), permission.getName(), permission.getPriority());
                parents.add(parent);
            }
            parents.sort(new AuthParentsComparator());
            return new GeneralResult<>(true, parents, "查询父权限成功");
        } catch (Exception e) {
            log.error("list permission parents fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<PermissionIcon>> listIcon(RequestContext requestContext) {
        try {
            // 判断是否是租户（自服务端）相关权限图标
            boolean tenant = false;
            if (null != requestContext.getTenant() && requestContext.getTenant() > 0) {
                tenant = true;
            }
            List<Permission> list = permissionRepository.listRoots(tenant);
            List<PermissionIcon> icons = new ArrayList<>();
            for (Permission permission : list) {
                PermissionIcon icon = new PermissionIcon(permission.getId(), permission.getIcon());
                icons.add(icon);
            }
            return new GeneralResult<>(true, icons, "查询成功!");
        } catch (Exception e) {
            log.error("list icon failure:", e);
            return new GeneralResult<>(false, "查询失败！");
        }
    }

    @Override
    public GeneralResult<List<PermissionBean>> list(Long parentId, String category, Boolean tenant) {
        try {
            List<Permission> permissions = permissionRepository.list(tenant, category, parentId);
            ArrayList<PermissionBean> permissionBeans = new ArrayList<>();
            // 重新构建返回权限实体
            for (Permission permit : permissions) {
                PermissionBean permissionBean = new PermissionBean(permit.getId(), permit.getName(), permit.getPath(),
                        permit.getIcon(), permit.getParentId(), permit.getPriority(), permit.getRouter(),
                        permit.getComponent(), permit.getRedirect(), permit.getMeta(), permit.getParams(),
                        permit.getCategory(), permit.getPluginsCode(), permit.getPluginsStatus(),permit.getButtonCode());
                permissionBeans.add(permissionBean);
            }
            if ("api".equalsIgnoreCase(category) || "button".equalsIgnoreCase(category)) {
                return new GeneralResult<>(true, permissionBeans, "查询成功!");
            }
            return new GeneralResult<>(true, getPermission(0L, permissionBeans), "查询成功!");
        } catch (Exception e) {
            log.error("list permissions failure:", e);
            return new GeneralResult<>(false, "查询失败！");
        }
    }

    /**
     * 对权限进行递归排序
     *
     * @param pid             父权限id
     * @param permissionBeans 权限实体集合
     * @return
     */
    private List<PermissionBean> getPermission(Long pid, List<PermissionBean> permissionBeans) {
        // 构建集合用于接收和返回递归后的权限
        List<PermissionBean> newPermissions = Lists.newArrayList();
        for (PermissionBean permissionBean : permissionBeans) {
            // 没有父权限id则递归结束
            if (null != permissionBean.getParentId() && permissionBean.getParentId().equals(pid)) {
                // 有父权限id则获取子权限集合再次递归调用，并设置排序后的子权限集合到父权限下
                List<PermissionBean> children = getPermission(permissionBean.getId(), permissionBeans);
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermitComparator());
                    permissionBean.setChildren(children);
                } else {
                    permissionBean.setChildren(null);
                }
                newPermissions.add(permissionBean);
            }
        }
        // 排序后返回
        newPermissions.sort(new PermitComparator());
        return newPermissions;
    }

    /**
     * 获取父权限
     *
     * @param parents    父权限集合
     * @param permission 目标权限
     * @return
     */
    private Map<Long, Permission> getParents(Map<Long, Permission> allPermissions, Map<Long, Permission> parents,
                                             Permission permission) {
        Long parentId = permission.getParentId();
        // 父权限id不存在时，递归出口，添加到父权限集合中，并返回
        if (null == parentId || 0L == parentId) {
            parents.put(permission.getId(), permission);
            return parents;
        } else {
            try {
                // 父权限id不为空，获取父权限，再次递归，并把当前权限添加到返回集合
                Permission parent = allPermissions.get(parentId);
                parents.put(permission.getId(), permission);
                return this.getParents(allPermissions, parents, parent);
            } catch (Exception e) {
                log.error("query permission parent list failure:", e);
                return null;
            }
        }
    }

    @Override
    public GeneralResult<Permission> getServicePermission(Long serviceId) {
        try {
            List<Permission> servicePermissions = permissionRepository.getServicePermissions(serviceId);
            Permission permission =
                    servicePermissions.stream().filter(permit -> "isServiceMenu".equals(permit.getProps())).findAny()
                            .orElse(servicePermissions.get(0));
            return new GeneralResult<>(true, permission, "获取服务菜单权限成功");
        } catch (Exception e) {
            log.error("get service permission error，" + e);
            return new GeneralResult<>(false, "获取服务菜单权限失败");
        }
    }

    @Override
    public GeneralResult<List<Permission>> getServicePermissions(List<Long> services) {
        try {
            // 获取相关服务权限集合
            List<Permission> servicePermissions = permissionRepository.getServicesPermissions(services);
            // 查询服务编码集合
            List<String> permissionCodes = new ArrayList<>();
            servicePermissions.forEach(permission -> {
                if (StringUtils.isNotBlank(permission.getServiceCodes())) {
                    permissionCodes.addAll(JSONArray.parseArray(permission.getServiceCodes(), String.class));
                }
            });
            // 获取所有自服务端权限
            List<Permission> allPermissions = permissionRepository.list(true);
            Map<Long, Permission> allPermissionMap =
                    allPermissions.stream().collect(Collectors.toMap(Permission::getId, Function.identity()));
            HashMap<Long, Permission> resultMap = new HashMap<>(servicePermissions.size(), 0.75F);
            // 获取相关服务对应的菜单权限
            for (Permission permission : servicePermissions) {
                // 查询父菜单权限(父菜单会放入result里面)
                getParents(allPermissionMap, resultMap, permission);
                // 获取当前服务权限相关的所有菜单权限（默认显示的和服务相关子权限）
                getRelatedPermission(resultMap, permission, permissionCodes, allPermissions);
                // 获取菜单权限父权限下默认显示权限
                List<Permission> brotherPermission = new ArrayList<>();
                Map<Long, List<Permission>> parentMap =
                        allPermissions.stream().collect(Collectors.groupingBy(Permission::getParentId));
                getBrotherPermission(permission.getParentId(), brotherPermission, parentMap);
                // 获取同级权限下默认显示权限
                List<Permission> alwaysPermissions = brotherPermission.stream()
                        .filter(permit -> "isAlwaysShow".equals(permit.getProps())).collect(Collectors.toList());
                // 使用map去重
                for (Permission alwaysPermission : alwaysPermissions) {
                    resultMap.put(alwaysPermission.getId(), alwaysPermission);
                }
            }
            ArrayList<Permission> permissions = new ArrayList<>(resultMap.values());
            return new GeneralResult<>(true, permissions, "获取服务相关权限成功。");
        } catch (Exception e) {
            log.error("get service permissions error," + e);
            return new GeneralResult<>(false, "获取服务相关权限失败。");
        }
    }

    /**
     * 获取授权服务相关权限集合
     *
     * @param resultMap    返回集合
     * @param permission   目标权限
     * @param serviceCodes 服务编码
     */
    private void getRelatedPermission(Map<Long, Permission> resultMap, Permission permission, List<String> serviceCodes,
                                      List<Permission> allPermissions) {
        // 获取所有菜单权限
        // 递归获取目标权限的子权限及下属子权限
        List<Permission> childrenPermission = getChildrenPermissions(permission.getId(), allPermissions);
        for (Permission childPermission : childrenPermission) {
            // 获取目标权限子权限中默认显示的权限及授权服务编码相关权限
            Map<Long,
                    Permission> permissionMap = childrenPermission.stream()
                    .filter(permit -> "isAlwaysShow".equals(permit.getProps())
                            || isContainsCode(serviceCodes, permit.getServiceCodes()))
                    .collect(Collectors.toMap(Permission::getId, Function.identity()));
            // 将符合的权限添加到返回集合中
            resultMap.putAll(permissionMap);
            // 对目标权限子权限进行递归，再次查询相关权限
            getRelatedPermission(resultMap, childPermission, serviceCodes, allPermissions);
        }
        // 判断目标权限是否是授权服务相关权限或者默认显示权限
        if ("isAlwaysShow".equals(permission.getProps())
                || isContainsCode(serviceCodes, permission.getServiceCodes())) {
            resultMap.put(permission.getId(), permission);
        }
    }

    /**
     * 判断权限服务code字符串是否存在集合中
     *
     * @param serviceCodes
     * @param codesString
     * @return 是否包含
     */
    private boolean isContainsCode(List<String> serviceCodes, String codesString) {
        return Optional.ofNullable(JSON.parseArray(codesString, String.class)).orElse(new ArrayList<>()).stream()
                .anyMatch(serviceCodes::contains);
    }

    private void getBrotherPermission(Long pid, List<Permission> result, Map<Long, List<Permission>> permissions) {
        List<Permission> children = Optional.ofNullable(permissions.get(pid)).orElse(new ArrayList<>());
        for (Permission child : children) {
            result.add(child);
            getBrotherPermission(child.getId(), result, permissions);
        }
    }

    /**
     * 递归获取子权限
     *
     * @param pid         父权限id
     * @param permissions 所有权限集合
     * @return
     */
    private List<Permission> getChildrenPermissions(Long pid, List<Permission> permissions) {
        // 创建用于接收和返回递归权限数据的集合
        List<Permission> newPermissions = Lists.newArrayList();
        for (Permission permission : permissions) {
            // 判断当前权限是否为目标权限子权限
            if (null != permission.getParentId() && permission.getParentId().equals(pid)) {
                // 递归查询，获取子权限的子权限
                List<Permission> children = getChildrenPermissions(permission.getId(), permissions);
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermissionComparator());
                    permission.setChilds(children);
                } else {
                    permission.setChilds(null);
                }
                // 添加到集合中
                newPermissions.add(permission);
            }
        }
        newPermissions.sort(new PermissionComparator());
        return newPermissions;
    }

    @Override
    public GeneralResult<JSONArray> getApiList() {
        try {
            Map<String, List<ApiInfo>> apiMap = new HashMap<>(8);
            Map<Object, Object> apiEntries = redisTemplate.opsForHash().entries("api_list");
            for (Map.Entry<Object, Object> entry : apiEntries.entrySet()) {
                String service = (String) entry.getKey();
                List<ApiInfo> apiInfos = JSONArray.parseArray(entry.getValue().toString(), ApiInfo.class);
                apiMap.put(service, apiInfos);
            }
            JSONArray result = new JSONArray();
            for (Map.Entry<String, List<ApiInfo>> entry : apiMap.entrySet()) {
                JSONObject serviceApi = new JSONObject();
                String service = entry.getKey();
                String serviceName = getServiceName(service);
                serviceApi.put("name", serviceName);
                serviceApi.put("code", service);
                Map<String, List<ApiInfo>> apiInfoMap =
                        entry.getValue().stream().collect(Collectors.groupingBy(ApiInfo::getTarget));
                JSONArray apiResult = new JSONArray();
                for (Map.Entry<String, List<ApiInfo>> infoEntry : apiInfoMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    String targetName = infoEntry.getKey();
                    List<ApiInfo> value = infoEntry.getValue();
                    jsonObject.put("name", targetName);
                    jsonObject.put("value", value);
                    apiResult.add(jsonObject);
                }
                serviceApi.put("value", apiResult);
                result.add(serviceApi);
            }
            return new GeneralResult<>(true, result, "查询api列表成功");
        } catch (Exception e) {
            log.error("get  module api list error,", e);
            return new GeneralResult<>(false, "查询api列表失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<JSONArray> getApiList(List<String> codes) {
        try {
            Map<String, List<ApiInfo>> apiMap = new HashMap<>(8);
            Map<Object, Object> apiEntries = redisTemplate.opsForHash().entries("api_list");
            for (Map.Entry<Object, Object> entry : apiEntries.entrySet()) {
                String service = (String) entry.getKey();
                if (codes.contains(service)){
                    List<ApiInfo> apiInfos = JSONArray.parseArray(entry.getValue().toString(), ApiInfo.class);
                    apiMap.put(service, apiInfos);
                }
            }
            JSONArray result = new JSONArray();
            for (Map.Entry<String, List<ApiInfo>> entry : apiMap.entrySet()) {
                JSONObject serviceApi = new JSONObject();
                String service = entry.getKey();
                String serviceName = getServiceName(service);
                serviceApi.put("name", serviceName);
                serviceApi.put("code", service);
                Map<String, List<ApiInfo>> apiInfoMap =
                        entry.getValue().stream().collect(Collectors.groupingBy(ApiInfo::getTarget));
                JSONArray apiResult = new JSONArray();
                for (Map.Entry<String, List<ApiInfo>> infoEntry : apiInfoMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    String targetName = infoEntry.getKey();
                    List<ApiInfo> value = infoEntry.getValue();
                    jsonObject.put("name", targetName);
                    jsonObject.put("value", value);
                    apiResult.add(jsonObject);
                }
                serviceApi.put("value", apiResult);
                result.add(serviceApi);
            }
            return new GeneralResult<>(true, result, "查询api列表成功");
        } catch (Exception e) {
            log.error("get  module api list error,", e);
            return new GeneralResult<>(false, "查询api列表失败:" + e.getMessage());
        }
    }

    private String getServiceName(String moduleName) {
        return BoCloudService.valueOf(moduleName.toUpperCase()).getProp();
    }

    @Override
    public GeneralResult<List<Permission>> listPermissionsByCondition(String condition) {
        JSONObject jsonObject = JSONObject.parseObject(condition);
        String category = jsonObject.getString("condition");
        if ("listByName".equals(category)) {
            String name = jsonObject.getString("name");
            List<Permission> permissions = permissionRepository.listByName(name);
            return new GeneralResult<>(true, permissions, "查询成功");
        }
        return new GeneralResult<>(false, "不支持这种查询方式");
    }

    /**
     * 插件绑定权限
     *
     * @param permissionIdList 权限id列表
     * @param pluginsCode      插件编码
     * @param tenant           是否是租户端的菜单页面
     * @param requestContext
     * @return
     */
    @Override
    public Result bindingPlugins(List<Long> permissionIdList, String pluginsCode, Boolean tenant,
                                 RequestContext requestContext) {
        try {
            Assert.notNull(permissionIdList, "permissionIdList can't be null");
            Assert.notNull(pluginsCode, "pluginsCode can't be null");
            // 判断该插件是否存在，如果不存在则直接返回
            Plugin dbPlugin = pluginRepository.getByCode(pluginsCode);
            Assert.notNull(dbPlugin, "未查询到该插件的信息!");
            // 解绑时将权限表中的插件名称改为"" 并将权限表中的插件状态修改为false
            this.permissionRepository.unBindingPlugins(pluginsCode, false, tenant);
            // 如果用户选择了权限表 那么需要进行绑定操作
            if (permissionIdList.size() > 0) {
                permissionIdList.forEach(permissionId -> {
                    Permission permission = this.permissionRepository.query(permissionId);
                    // 该权限菜单没有绑定插件 才可以进行绑定操作
                    if (StringUtils.isBlank(permission.getPluginsCode())) {
                        this.permissionRepository.bindingPlugins(Collections.singletonList(permission.getId()),
                                pluginsCode, dbPlugin.getStatus());
                    }
                    if (permission.getParentId() != 0) {
                        this.bindParentPermission(permission.getId(), pluginsCode, dbPlugin.getStatus());
                    }
                });
            }
            return new GeneralResult<>(true, "插件绑定权限成功");
        } catch (Exception e) {
            log.error("plugins binding permission fail:", e);
            return new GeneralResult<>(false, "插件绑定权限失败:" + e.getMessage());
        }
    }

    private void bindParentPermission(Long id, String pluginsCode, Boolean status) {
        try {
            Permission permission = this.permissionRepository.query(id);
            // 该权限菜单没有绑定插件 才可以进行绑定操作
            if ("".equals(permission.getPluginsCode())) {
                this.permissionRepository.bindingPlugins(Collections.singletonList(permission.getId()), pluginsCode,
                        status);
            }
            if (permission.getParentId() != 0) {
                this.bindParentPermission(permission.getParentId(), pluginsCode, status);
            }
        } catch (Exception e) {
            log.error("绑定权限失败！" + e);
        }
    }

}
