package com.bocloud.sms.repository;

import com.bocloud.sms.entity.Plugin;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 插件
 *
 * @author zhangyf
 * @since 2022-05-19
 */
@Slf4j
@Repository
public class PluginRepository extends BasicGenericDao<Plugin, Long> {

    private final CurrentService currentService;

    public PluginRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService currentService) {
        super(jdbcTemplate, npJdbcTemplate, currentService);
        this.currentService = currentService;
    }

    /**
     * 查询plugin总数
     *
     * @param params 参数
     * @return plugin总数
     */
    public int count(List<Param> params) {
        List<String> plugins = LicenseDaemon.instance(currentService).content().getPlugins();
        List<String> globals = LicenseDaemon.instance(currentService).content().getGlobals();
        String sql = "select count(1) from plugin where is_deleted = 0 and (code in (:plugins) or code not in (:globals)) ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("plugins", plugins);
        paramMap.put("globals", globals);
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询plugin列表
     *
     * @param pager 分页
     * @return plugin集合
     */
    public List<Plugin> list(Pager pager) {
        List<String> plugins = LicenseDaemon.instance(currentService).content().getPlugins();
        List<String> globals = LicenseDaemon.instance(currentService).content().getGlobals();
        String sql = "select * from plugin where is_deleted = 0 and (code in (:plugins) or code not in (:globals)) ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("plugins", plugins);
        paramMap.put("globals", globals);
        sql = getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(Plugin.class, sql, paramMap);
    }

    /**
     * 根据插件编码集合获取已开启插件列表
     *
     * @param plugin 插件编码集合
     * @return plugin集合
     */
    public List<Plugin> listByCode(List<String> plugin) {
        if (plugin.isEmpty()) {
            log.warn("param plugin is empty");
            return Collections.emptyList();
        }
        List<String> plugins = LicenseDaemon.instance(currentService).content().getPlugins();
        List<String> globals = LicenseDaemon.instance(currentService).content().getGlobals();
        String sql = "select * from plugin where is_deleted = 0 and (code in (:plugins) or code not in (:globals)) and code in (:plugin) ";
        Map<String, Object> paramMap = MapTools.simpleMap("plugin", plugin);
        paramMap.put("plugins", plugins);
        paramMap.put("globals", globals);
        return this.list(Plugin.class, sql, paramMap);
    }

    /**
     * 根据插件唯一Code进行查询
     *
     * @param code 插件编码
     * @return 插件
     */
    public Plugin getByCode(String code) {
        return getByCode(code, null);
    }

    /**
     * 根据插件唯一Code进行查询
     *
     * @param code    插件编码
     * @param version 版本
     * @return 插件
     */
    public Plugin getByCode(String code, String version) {
        String sql = "select * from plugin where is_deleted = 0 and code = :code ";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        if (StringUtils.isNotBlank(version)) {
            sql += " and version = :version ";
            paramMap.put("version", version);
        }
        List<Plugin> plugin = this.list(Plugin.class, sql, paramMap);
        return ListTool.isEmpty(plugin) ? null : plugin.get(0);
    }

    /**
     * 根据插件name进行查询
     *
     * @param name    插件名称
     * @param version 版本
     * @return 插件
     */
    public Plugin getByName(String name, String version) {
        String sql = "select * from plugin where is_deleted = 0 and name = :name and version = :version ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("version", version);
        List<Plugin> plugin = this.list(Plugin.class, sql, paramMap);
        return ListTool.isEmpty(plugin) ? null : plugin.get(0);
    }

    /**
     * 判断插件是否存在同名
     *
     * @param name 插件名称
     * @return 插件名称是否已存在
     */
    public boolean existsSameName(String name) {
        String sql = "select count(1) from plugin where is_deleted = 0 and name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }

    /**
     * 判断插件编码是否重复
     *
     * @param code 插件编码
     * @return 编码是否已经存在
     */
    public boolean existsSameCode(String code) {
        String sql = "select count(1) from plugin where is_deleted = 0 and code = :code";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }
}
