package com.bocloud.sms.repository;

import com.bocloud.sms.entity.UserProject;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;

/**
 * 用户项目关联关系
 *
 * @author zhangyf
 * @since 2022-03-02
 */
@Repository
public class UserProjectRepository extends BasicGenericDao<UserProject, Long> {

    @Autowired
    public UserProjectRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 删除用户项目关联关系
     *
     * @param userId 用户ID
     * @return
     */
    public boolean unBindProject(Long userId) {
        String sql = "delete from user_project where user_id = :userId";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        return super.execute(sql, params) > 0;
    }


    /**
     * 删除用户项目关联关系
     *
     * @param projectIds 项目ID集合
     * @return
     */
    public boolean unBindProject(List<Long> projectIds) {
        String sql = "delete from user_project where project_id in (:projectIds)";
        Map<String, Object> params = MapTools.simpleMap("projectIds", projectIds);
        return super.execute(sql, params) > 0;
    }

    /**
     * 删除用户项目关联关系
     *
     * @param projectId 项目ID
     * @return
     */
    public boolean unBindProject(Long projectId, Boolean projectAdmin) throws DataBaseException {
        String sql = "delete from user_project where project_id = :projectId";
        Map<String, Object> params = MapTools.simpleMap("projectId", projectId);
        if (null != projectAdmin) {
            sql += " and project_admin = :projectAdmin";
            params.put("projectAdmin", projectAdmin);
        }
        return super.execute(sql, params) > 0;
    }

    /**
     * 通过 projectId 查询 用户项目关系
     *
     * @param projectId 项目ID
     * @return
     */
    public List<UserProject> listUserProject(Long projectId) throws DataBaseException {
        String sql = "select * from user_project where project_id = :projectId";
        Map<String, Object> params = MapTools.simpleMap("projectId", projectId);
        return super.list(UserProject.class, sql, params);
    }

    /**
     * 通过 userId 查询 用户项目关系
     *
     * @param userId 用户ID
     * @return
     */
    public List<UserProject> listUserProjectByUserId(Long userId) throws DataBaseException {
        String sql = "select * from user_project where user_id = :userId";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        return super.list(UserProject.class, sql, params);
    }

    /**
     * 通过 userId 和 projectId 查询用户项目关系
     *
     * @param userId    用户ID
     * @param projectId 项目ID
     * @return
     */
    public UserProject getUserProject(Long userId, Long projectId) throws DataBaseException {
        String sql = "select * from user_project where user_id = :userId and project_id = :projectId ";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        params.put("projectId", projectId);
        List<UserProject> list = super.list(UserProject.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one user project in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * @param userIds 用户ID集
     * @return
     */
    public List<UserProject> listUsersGlobalProjects(List<Long> userIds) {
        String sql = "select distinct b.* from  user_project b left join project a  on a.id = b.project_id where a.is_deleted = 0 and b.user_id in (:userIds) and (a.tenant_id is null or a.tenant_id = 0 )";
        Map<String, Object> paramMap = MapTools.simpleMap("userIds", userIds);
        return super.list(UserProject.class, sql, paramMap);
    }

}
